@@ -2,6 +2,10 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import numpy as np
+import torch
+import torch.nn as nn
+import torch.nn.functional as F
+import numpy as np
 
 class ImplicitMLPLayer(nn.Module):
     def __init__(self, in_features, out_features, bias=True,
@@ -21,7 +25,7 @@ class ImplicitMLPLayer(nn.Module):
         self.film_conditioning = film_conditioning
         self.concat_conditioning = concat_conditioning
         if concat_conditioning:
-            self.in_features += concat_conditioning
+            self.in_features += concat_conditioning 
 
         self.linear = nn.Linear(self.in_features, out_features, bias=bias)
 
@@ -72,11 +76,11 @@ class ImplicitMLPLayer(nn.Module):
             if init_method["bias"] == "none":
                 pass
     
-    def forward(self, input, z=None, gamma=None, beta=None, delta=None, progress=None):
+    def forward(self, layer_input, z=None, gamma=None, beta=None, delta=None, progress=None):
         if self.concat_conditioning:
-            if z.shape[1] !=  input.shape[1]: 
-                z = z.repeat(1, input.shape[1], 1)
-            input = torch.cat((input, z), dim=-1)
+            if z.shape[1] !=  layer_input.shape[1]: 
+                z = z.repeat(1, layer_input.shape[1], 1)
+            layer_input = torch.cat((layer_input, z), dim=-1)
 
         if self.film_conditioning:
             self.feat_multiplier = gamma[:, :,  :self.out_features] + self.omega_0
@@ -86,7 +90,7 @@ class ImplicitMLPLayer(nn.Module):
             self.feat_multiplier = self.omega_0
             self.feat_bias = 0
 
-        output = self.activation((self.feat_multiplier * self.linear(input)) + self.feat_bias)
+        output = self.activation((self.feat_multiplier * self.linear(layer_input)) + self.feat_bias)
         if delta is not None:
             output = output * delta
         return output
@@ -170,19 +174,19 @@ class IMNET(nn.Module):
         self.net = []
 
         if siren_first:
-            self.net.append(ImplicitMLPLayer(in_features, initial_hidden, activation="sine", omega_0=30,
+            self.net.append(ImplicitMLPLayer(1, initial_hidden, activation="sine", omega_0=30,
                                             concat_conditioning=z_size, init_method={"weights": 'siren_first', "bias": "polar"}))
         else:
-            self.net.append(ImplicitMLPLayer(in_features, initial_hidden, activation="relu",
+            self.net.append(ImplicitMLPLayer(1, initial_hidden, activation="relu",
                                         concat_conditioning=z_size, init_method=init_method))
         
         for i, layer in enumerate(range(num_layers-1)):
             if siren_hidden:
                 self.net.append(ImplicitMLPLayer(initial_hidden//(2**i), initial_hidden//(2**(i+1)), activation="sine", omega_0=30,
-                                                concat_conditioning=z_size, init_method={"weights": 'siren', "bias": "polar"}))
+                                                concat_conditioning=z_size + 1, init_method={"weights": 'siren', "bias": "polar"}))
             else:
                 self.net.append(ImplicitMLPLayer(initial_hidden//(2**i), initial_hidden//(2**(i+1)), activation="relu",
-                                                concat_conditioning=z_size, init_method=init_method))
+                                                concat_conditioning=z_size + 1, init_method=init_method))
         
         if siren_final:
             self.net.append(ImplicitMLPLayer(initial_hidden//(2**(i+1)), out_features,
@@ -197,32 +201,33 @@ class IMNET(nn.Module):
     
     def forward(self, coordinates, z):
         coordinates = coordinates * self.coord_multi
+        coordinates = coordinates.clone().detach().requires_grad_(True) # allows to take derivative w.r.t. input
+        z = z.repeat(1, coordinates.shape[1], 1)
         output = self.net[0](coordinates, z=z)
         for layer in self.net[1:]:
             if layer.concat_conditioning:
-                output = layer(output, z=z)
+                output = layer(output, z=torch.cat((z, coordinates), dim=-1))
             else:
                 output = layer(output)
-
         return output
 
 class piGAN(nn.Module):
     def __init__(self, in_features, out_features, bias=True,
-                num_mapping_layers=3, num_siren_layers=8, num_hidden_mapping=256, num_hidden_siren=256, 
+                num_mapping_layers=3, num_INR_layers=8, num_hidden_mapping=256, num_hidden_INR=256, 
                 z_size=256, first_omega_0=600, hidden_omega_0=30):
         super().__init__()
         self.mapping_net = PiGANMappingNetwork(z_size, num_hidden_mapping, num_hidden_mapping*2, depth = num_mapping_layers)
         self.net = []
-        self.net.append(ImplicitMLPLayer(in_features, num_hidden_siren, bias=True,
+        self.net.append(ImplicitMLPLayer(in_features, num_hidden_INR, bias=True,
                         omega_0=first_omega_0, w_norm=True, activation="sine", 
                         film_conditioning=True, concat_conditioning=0,
                         init_method={"weights": 'siren_first', "bias": "polar"}))
-        for i in range(num_siren_layers-1):
-            self.net.append(ImplicitMLPLayer(num_hidden_siren, num_hidden_siren, bias=True,
+        for i in range(num_INR_layers-1):
+            self.net.append(ImplicitMLPLayer(num_hidden_INR, num_hidden_INR, bias=True,
                             omega_0=hidden_omega_0, w_norm=True, activation="sine", 
                             film_conditioning=True, concat_conditioning=0,
                             init_method={"weights": 'siren', "bias": "polar"}))
-        self.net.append(ImplicitMLPLayer(num_hidden_siren, out_features, bias=True,
+        self.net.append(ImplicitMLPLayer(num_hidden_INR, out_features, bias=True,
                 omega_0=1, w_norm=True, activation="none", 
                 film_conditioning=False, concat_conditioning=0,
                 init_method={"weights": 'siren_omega', "omega":30, "bias": "none"}))
@@ -301,31 +306,30 @@ class SIREN_prog(nn.Module):
 
 class piGAN_prog(nn.Module):
     def __init__(self, in_features, out_features, bias=True,
-                num_mapping_layers=3, num_siren_layers=8, num_hidden_mapping=256, num_hidden_siren=256, 
+                num_mapping_layers=3, num_INR_layers=8, num_hidden_mapping=256, num_hidden_INR=256, 
                 z_size=256, first_omega_0=3000, hidden_omega_0=30, total_epochs=5000, num_groups=256):
         super().__init__()
         self.mapping_net = PiGANMappingNetwork(z_size, num_hidden_mapping, num_hidden_mapping*2, depth = num_mapping_layers)
         self.net = []
-        self.net.append(ImplicitMLPLayer(in_features, num_hidden_siren, bias=True,
+        self.net.append(ImplicitMLPLayer(in_features, num_hidden_INR, bias=True,
                         omega_0=first_omega_0, w_norm=True, activation="sine", 
                         film_conditioning=True, concat_conditioning=0, omega_uniform=True,
                         init_method={"weights": 'siren_first', "bias": "polar"}))
-        for i in range(num_siren_layers-1):
-            self.net.append(ImplicitMLPLayer(num_hidden_siren, num_hidden_siren, bias=True,
+        for i in range(num_INR_layers-1):
+            self.net.append(ImplicitMLPLayer(num_hidden_INR, num_hidden_INR, bias=True,
                             omega_0=hidden_omega_0, w_norm=True, activation="sine", 
                             film_conditioning=True, concat_conditioning=0, omega_uniform=False,
                             init_method={"weights": 'siren', "bias": "polar"}))
-        self.net.append(ImplicitMLPLayer(num_hidden_siren, out_features, bias=True,
+        self.net.append(ImplicitMLPLayer(num_hidden_INR, out_features, bias=True,
                 omega_0=1, w_norm=True, activation="none", 
                 film_conditioning=False, concat_conditioning=0,
                 init_method={"weights": 'siren_omega', "omega":30, "bias": "none"}))
         self.net = nn.Sequential(*self.net)
 
-        # self.num_hidden = num_hidden_siren
         self.total_epochs = total_epochs
         self.num_groups = num_groups
-        self.switch_layer = nn.Parameter(torch.zeros(num_hidden_siren), requires_grad=False)
-        self.groups = [(i*(num_hidden_siren//num_groups), (i+1)*(num_hidden_siren//num_groups)) for i in range(num_groups)]
+        self.switch_layer = nn.Parameter(torch.zeros(num_hidden_INR), requires_grad=False)
+        self.groups = [(i*(num_hidden_INR//num_groups), (i+1)*(num_hidden_INR//num_groups)) for i in range(num_groups)]
         self.buffer_time = (total_epochs // 2) // ((2 * num_groups))
         self.buffer_group = 0
         self.epoch = 0
@@ -376,7 +380,7 @@ class piGAN_prog(nn.Module):
 
 class piGAN_custom(nn.Module):
     def __init__(self, in_features, out_features, bias=True,
-                num_mapping_layers=3, num_siren_layers=8, num_hidden_mapping=256, num_hidden_siren=256, 
+                num_mapping_layers=3, num_INR_layers=8, num_hidden_mapping=256, num_hidden_INR=256, 
                 z_size=256, first_omega_0=600, hidden_omega_0=30, 
                 activations= ["sine", "sine", "none"], # "sine", "relu", "none"
                 conditioning_method = "film", # "concat", "film", "both"
@@ -424,7 +428,7 @@ class piGAN_custom(nn.Module):
         if self.film_conditioning:
             self.film_mapping_net = PiGANMappingNetwork(z_size,
                                                     num_hidden_mapping, 
-                                                    num_hidden_siren*2,
+                                                    num_hidden_INR*2,
                                                     depth = num_mapping_layers)
         if self.concat_conditioning_first:
             self.concat_mapping_net = ConcatMappingNetwork(z_size,
@@ -433,33 +437,33 @@ class piGAN_custom(nn.Module):
                                                     depth = num_mapping_layers)
 
         self.net = []
-        self.net.append(ImplicitMLPLayer(in_features, num_hidden_siren, bias=True,
+        self.net.append(ImplicitMLPLayer(in_features, num_hidden_INR, bias=True,
                         omega_0=first_omega_0, w_norm=False, activation=activations[0], 
                         film_conditioning=self.film_conditioning, concat_conditioning=z_size*self.concat_conditioning_first,
                         init_method=init_methods[0]))
         
-        last_hidden = num_hidden_siren
-        for i in range(num_siren_layers-1):
+        last_hidden = num_hidden_INR
+        for i in range(num_INR_layers-1):
             if self.network_shape == "shrinking":
-                num_hidden_siren = num_hidden_siren // 2
+                num_hidden_INR = num_hidden_INR // 2
 
             if self.concat_conditioning_middle and i == 3:
-                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_siren, bias=True,
+                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_INR, bias=True,
                                 omega_0=hidden_omega_0, w_norm=False, activation=activations[1], 
-                                film_conditioning=self.film_conditioning, concat_conditioning=z_size+in_features,
+                                film_conditioning=self.film_conditioning, concat_conditioning=z_size+1,
                                 init_method=init_methods[1]))
             elif self.concat_conditioning_all:
-                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_siren, bias=True,
+                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_INR, bias=True,
                                 omega_0=hidden_omega_0, w_norm=False, activation=activations[1], 
-                                film_conditioning=self.film_conditioning, concat_conditioning=z_size+in_features,
+                                film_conditioning=self.film_conditioning, concat_conditioning=z_size+1,
                                 init_method=init_methods[1]))
             else:
-                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_siren, bias=True,
+                self.net.append(ImplicitMLPLayer(last_hidden, num_hidden_INR, bias=True,
                                 omega_0=hidden_omega_0, w_norm=False, activation=activations[1], 
                                 film_conditioning=self.film_conditioning, concat_conditioning=0,
                                 init_method=init_methods[1]))
 
-            last_hidden = num_hidden_siren
+            last_hidden = num_hidden_INR
 
         self.net.append(ImplicitMLPLayer(last_hidden, out_features, bias=True,
                 omega_0=1, w_norm=False, activation=activations[2], 
@@ -479,14 +483,14 @@ class piGAN_custom(nn.Module):
             gamma, beta = self.film_mapping_net(z)
         elif self.concat_conditioning_first:
             concat = self.concat_mapping_net(z)
-        
         coordinates = coordinates.clone().detach().requires_grad_(True) # allows to take derivative w.r.t. input
+        concat = concat.repeat(1, coordinates.shape[1], 1)
         output = self.net[0](coordinates, gamma=gamma, beta=beta, z=concat)
         for layer in self.net[1:]:
             if layer.film_conditioning:
                 output = layer(output, gamma=gamma, beta=beta)
             elif layer.concat_conditioning:
-                output = layer(output, z=torch.cat(concat, coordinates))
+                output = layer(output, z=torch.cat((concat, coordinates), dim=-1))
             else:
                 output = layer(output)
-        return output
+        return output
\ No newline at end of file
